package org.roboid.android;

import org.roboid.robot.DeviceDataChangedListener;
import org.roboid.robot.Robot;
import org.roboid.robot.impl.RoboidImpl;

/**
 * @author akaii@kw.ac.kr (Kwang-Hyun Park)
 */
abstract class AbstractRoboid extends RoboidImpl
{
	AbstractRoboid(Robot robot, int size)
	{
		super(size);
	}
	
	AbstractRoboid(int size, int tag)
	{
		super(size, tag);
	}
	
	void handleSensorySimulacrum(Robot robot, byte[] simulacrum, long timestamp)
	{
		if(decodeSensorySimulacrum(simulacrum))
		{
			synchronized(mDeviceDataChangedListeners)
			{
				for(DeviceDataChangedListener listener : mDeviceDataChangedListeners)
					notifySensoryDataChanged(listener, robot, timestamp);
			}
		}
	}
	
	void handleMotoringSimulacrum(Robot robot, byte[] simulacrum, long timestamp)
	{
		if(decodeMotoringSimulacrum(simulacrum))
		{
			synchronized(mDeviceDataChangedListeners)
			{
				for(DeviceDataChangedListener listener : mDeviceDataChangedListeners)
					notifyMotoringDataChanged(listener, robot, timestamp);
			}
		}
	}
	
	abstract boolean decodeSensorySimulacrum(byte[] simulacrum);
	abstract void notifySensoryDataChanged(DeviceDataChangedListener listener, Robot robot, long timestamp);
	
	boolean decodeMotoringSimulacrum(byte[] simulacrum)
	{
		return false;
	}
	
	void notifyMotoringDataChanged(DeviceDataChangedListener listener, Robot robot, long timestamp)
	{
	}
	
	byte[] encodeMotoringSimulacrum()
	{
		return null;
	}
}
