package org.roboid.android;

import kr.robomation.physical.Hamster;

import org.roboid.robot.DataType;
import org.roboid.robot.Device;
import org.roboid.robot.DeviceDataChangedListener;
import org.roboid.robot.DeviceType;
import org.roboid.robot.Robot;

/**
 * @author akaii@kw.ac.kr (Kwang-Hyun Park)
 */
final class HamsterRoboid extends AbstractRoboid
{
	private int mReadSensoryFlag;
	private int mReadMotoringFlag;
	private int mWriteMotoringFlag;
	private final byte[] mMotoringSimulacrum = new byte[36];
	private final Object[] mReadBuffer = new Object[27];
	private final Object[] mWriteBuffer = new Object[16];
	
	HamsterRoboid()
	{
		super(27, 0x00400000);
		
		Object[] readBuffer = mReadBuffer;
		Object[] writeBuffer = mWriteBuffer;
		
		Device device = addDevice(0, DeviceType.EFFECTOR, Hamster.LEFT_WHEEL, "LeftWheel", DataType.INTEGER, 1, 0);
		readBuffer[0] = getReadData(device);
		writeBuffer[0] = getWriteData(device);
		
		device = addDevice(1, DeviceType.EFFECTOR, Hamster.RIGHT_WHEEL, "RightWheel", DataType.INTEGER, 1, 0);
		readBuffer[1] = getReadData(device);
		writeBuffer[1] = getWriteData(device);
		
		device = addDevice(2, DeviceType.EFFECTOR, Hamster.BUZZER, "Buzzer", DataType.FLOAT, 1, 0.0f);
		readBuffer[2] = getReadData(device);
		writeBuffer[2] = getWriteData(device);
		
		device = addDevice(3, DeviceType.EFFECTOR, Hamster.OUTPUT_A, "OutputA", DataType.INTEGER, 1, 0);
		readBuffer[3] = getReadData(device);
		writeBuffer[3] = getWriteData(device);
		
		device = addDevice(4, DeviceType.EFFECTOR, Hamster.OUTPUT_B, "OutputB", DataType.INTEGER, 1, 0);
		readBuffer[4] = getReadData(device);
		writeBuffer[4] = getWriteData(device);
		
		device = addDevice(5, DeviceType.COMMAND, Hamster.TOPOLOGY, "Topology", DataType.INTEGER, 1, 0);
		readBuffer[5] = getReadData(device);
		writeBuffer[5] = getWriteData(device);
		
		device = addDevice(6, DeviceType.COMMAND, Hamster.LEFT_LED, "LeftLed", DataType.INTEGER, 1, 0);
		readBuffer[6] = getReadData(device);
		writeBuffer[6] = getWriteData(device);

		device = addDevice(7, DeviceType.COMMAND, Hamster.RIGHT_LED, "RightLed", DataType.INTEGER, 1, 0);
		readBuffer[7] = getReadData(device);
		writeBuffer[7] = getWriteData(device);
		
		device = addDevice(8, DeviceType.COMMAND, Hamster.NOTE, "Note", DataType.INTEGER, 1, 0);
		readBuffer[8] = getReadData(device);
		writeBuffer[8] = getWriteData(device);
		
		device = addDevice(9, DeviceType.COMMAND, Hamster.LINE_TRACER_MODE, "LineTracerMode", DataType.INTEGER, 1, 0);
		readBuffer[9] = getReadData(device);
		writeBuffer[9] = getWriteData(device);
		
		device = addDevice(10, DeviceType.COMMAND, Hamster.LINE_TRACER_SPEED, "LineTracerSpeed", DataType.INTEGER, 1, 5);
		readBuffer[10] = getReadData(device);
		writeBuffer[10] = getWriteData(device);
		
		device = addDevice(11, DeviceType.COMMAND, Hamster.IO_MODE_A, "IoModeA", DataType.INTEGER, 1, 0);
		readBuffer[11] = getReadData(device);
		writeBuffer[11] = getWriteData(device);
		
		device = addDevice(12, DeviceType.COMMAND, Hamster.IO_MODE_B, "IoModeB", DataType.INTEGER, 1, 0);
		readBuffer[12] = getReadData(device);
		writeBuffer[12] = getWriteData(device);

		device = addDevice(13, DeviceType.COMMAND, Hamster.CONFIG_PROXIMITY, "ConfigProximity", DataType.INTEGER, 1, 2);
		readBuffer[13] = getReadData(device);
		writeBuffer[13] = getWriteData(device);
		
		device = addDevice(14, DeviceType.COMMAND, Hamster.CONFIG_GRAVITY, "ConfigGravity", DataType.INTEGER, 1, 0);
		readBuffer[14] = getReadData(device);
		writeBuffer[14] = getWriteData(device);
		
		device = addDevice(15, DeviceType.COMMAND, Hamster.CONFIG_BAND_WIDTH, "ConfigBandWidth", DataType.INTEGER, 1, 3);
		readBuffer[15] = getReadData(device);
		writeBuffer[15] = getWriteData(device);

		device = addDevice(16, DeviceType.SENSOR, Hamster.SIGNAL_STRENGTH, "SignalStrength", DataType.INTEGER, 1, 0);
		readBuffer[16] = getReadData(device);
		
		device = addDevice(17, DeviceType.SENSOR, Hamster.LEFT_PROXIMITY, "LeftProximity", DataType.INTEGER, 1, 0);
		readBuffer[17] = getReadData(device);
		
		device = addDevice(18, DeviceType.SENSOR, Hamster.RIGHT_PROXIMITY, "RightProximity", DataType.INTEGER, 1, 0);
		readBuffer[18] = getReadData(device);
		
		device = addDevice(19, DeviceType.SENSOR, Hamster.LEFT_FLOOR, "LeftFloor", DataType.INTEGER, 1, 0);
		readBuffer[19] = getReadData(device);
		
		device = addDevice(20, DeviceType.SENSOR, Hamster.RIGHT_FLOOR, "RightFloor", DataType.INTEGER, 1, 0);
		readBuffer[20] = getReadData(device);
		
		device = addDevice(21, DeviceType.SENSOR, Hamster.ACCELERATION, "Acceleration", DataType.INTEGER, 3, 0);
		readBuffer[21] = getReadData(device);
		
		device = addDevice(22, DeviceType.SENSOR, Hamster.LIGHT, "Light", DataType.INTEGER, 1, 0);
		readBuffer[22] = getReadData(device);
		
		device = addDevice(23, DeviceType.SENSOR, Hamster.TEMPERATURE, "Temperature", DataType.INTEGER, 1, 0);
		readBuffer[23] = getReadData(device);
		
		device = addDevice(24, DeviceType.SENSOR, Hamster.INPUT_A, "InputA", DataType.INTEGER, 1, 0);
		readBuffer[24] = getReadData(device);
		
		device = addDevice(25, DeviceType.SENSOR, Hamster.INPUT_B, "InputB", DataType.INTEGER, 1, 0);
		readBuffer[25] = getReadData(device);
		
		device = addDevice(26, DeviceType.EVENT, Hamster.LINE_TRACER_STATE, "LineTracerState", DataType.INTEGER, 2, 0);
		readBuffer[26] = getReadData(device);
	}
	
	@Override
	public String getId()
	{
		return Hamster.ID;
	}
	
	@Override
	public String getName()
	{
		return "Hamster";
	}
	
	@Override
	protected void onMotoringDeviceWritten(Device device)
	{
		switch(device.getId())
		{
		case Hamster.TOPOLOGY:
			mWriteMotoringFlag |= 0x02000000;
			break;
		case Hamster.LEFT_LED:
			mWriteMotoringFlag |= 0x01000000;
			break;
		case Hamster.RIGHT_LED:
			mWriteMotoringFlag |= 0x00800000;
			break;
		case Hamster.NOTE:
			mWriteMotoringFlag |= 0x00400000;
			break;
		case Hamster.LINE_TRACER_MODE:
			mWriteMotoringFlag |= 0x00200000;
			break;
		case Hamster.LINE_TRACER_SPEED:
			mWriteMotoringFlag |= 0x00100000;
			break;
		case Hamster.IO_MODE_A:
			mWriteMotoringFlag |= 0x00080000;
			break;
		case Hamster.IO_MODE_B:
			mWriteMotoringFlag |= 0x00040000;
			break;
		case Hamster.CONFIG_PROXIMITY:
			mWriteMotoringFlag |= 0x00020000;
			break;
		case Hamster.CONFIG_GRAVITY:
			mWriteMotoringFlag |= 0x00010000;
			break;
		case Hamster.CONFIG_BAND_WIDTH:
			mWriteMotoringFlag |= 0x00008000;
			break;
		}
	}
	
	@Override
	protected boolean decodeSensorySimulacrum(byte[] simulacrum)
	{
		if(simulacrum == null || simulacrum.length < 23) return false;
		
		int flag = 0;
		synchronized(mReadLock)
		{
			Object[] buffer = mReadBuffer;

			// signal strength
			((int[])buffer[16])[0] = simulacrum[6]; // byte

			// left proximity
			((int[])buffer[17])[0] = simulacrum[7] & 0xff; // unsigned byte

			// right proximity
			((int[])buffer[18])[0] = simulacrum[8] & 0xff; // unsigned byte

			// left floor
			((int[])buffer[19])[0] = simulacrum[9] & 0xff; // unsigned byte

			// right floor
			((int[])buffer[20])[0] = simulacrum[10] & 0xff; // unsigned byte

			// acceleration
			int[] t = (int[])buffer[21];
			int v = simulacrum[11] << 8; // short
			v |= simulacrum[12] & 0xff;
			t[0] = v;
			v = simulacrum[13] << 8; // short
			v |= simulacrum[14] & 0xff;
			t[1] = v;
			v = simulacrum[15] << 8; // short
			v |= simulacrum[16] & 0xff;
			t[2] = v;

			// light
			v = (simulacrum[17] & 0xff) << 8; // unsigned short
			v |= simulacrum[18] & 0xff;
			((int[])buffer[22])[0] = v;

			// temperature
			((int[])buffer[23])[0] = simulacrum[19]; // byte

			// input a
			((int[])buffer[24])[0] = simulacrum[20] & 0xff; // unsigned byte

			// input b
			((int[])buffer[25])[0] = simulacrum[21] & 0xff; // unsigned byte

			// line tracer state
			if((simulacrum[5] & 0x10) != 0)
			{
				flag |= 0x00000010;
				t = (int[])buffer[26];
				t[0] = simulacrum[23] & 0xff; // unsigned byte
				t[1] = simulacrum[24] & 0xff; // unsigned byte
			}
		}
		for(int i = 16; i < 26; ++i)
			fire(i);
		if((flag & 0x00000010) != 0)
			fire(26);
		mReadSensoryFlag = flag;
		return true;
	}
	
	@Override
	protected void notifySensoryDataChanged(DeviceDataChangedListener listener, Robot robot, long timestamp)
	{
		Device[] devices = mDevices;
		Object[] buffer = mReadBuffer;
		
		for(int i = 16; i < 26; ++i)
			listener.onDeviceDataChanged(robot, devices[i], buffer[i], timestamp);
		if((mReadSensoryFlag & 0x00000010) != 0)
			listener.onDeviceDataChanged(robot, devices[26], buffer[26], timestamp);
	}
	
	@Override
	protected boolean decodeMotoringSimulacrum(byte[] simulacrum)
	{
		if(simulacrum == null || simulacrum.length < 25) return false;
		
		int flag = 0;
		synchronized(mReadLock)
		{
			Object[] buffer = mReadBuffer;

			// left wheel
			((int[])buffer[0])[0] = simulacrum[6]; // byte

			// right wheel
			((int[])buffer[1])[0] = simulacrum[7]; // byte

			// buzzer
			int v = simulacrum[8] << 24; // integer
			v |= (simulacrum[9] & 0xff) << 16;
			v |= (simulacrum[10] & 0xff) << 8;
			v |= simulacrum[11] & 0xff;
			((float[])buffer[2])[0] = (float)(v / 100.0f);

			// output a
			((int[])buffer[3])[0] = simulacrum[12] & 0xff; // unsigned byte

			// output b
			((int[])buffer[4])[0] = simulacrum[13] & 0xff; // unsigned byte

			// topology
			int index = 15;
			if((simulacrum[2] & 0x02) != 0)
			{
				flag |= 0x02000000;
				((int[])buffer[5])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// left led
			++ index;
			if((simulacrum[2] & 0x01) != 0)
			{
				flag |= 0x01000000;
				((int[])buffer[6])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// right led
			++ index;
			if((simulacrum[3] & 0x80) != 0)
			{
				flag |= 0x00800000;
				((int[])buffer[7])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// note
			++ index;
			if((simulacrum[3] & 0x40) != 0)
			{
				flag |= 0x00400000;
				((int[])buffer[8])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// line tracer mode
			++ index;
			if((simulacrum[3] & 0x20) != 0)
			{
				flag |= 0x00200000;
				((int[])buffer[9])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// line tracer speed
			++ index;
			if((simulacrum[3] & 0x10) != 0)
			{
				flag |= 0x00100000;
				((int[])buffer[10])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// io mode a
			++ index;
			if((simulacrum[3] & 0x08) != 0)
			{
				flag |= 0x00080000;
				((int[])buffer[11])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// io mode b
			++ index;
			if((simulacrum[3] & 0x04) != 0)
			{
				flag |= 0x00040000;
				((int[])buffer[12])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// config proximity
			++ index;
			if((simulacrum[3] & 0x02) != 0)
			{
				flag |= 0x00020000;
				((int[])buffer[13])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// config gravity
			++ index;
			if((simulacrum[3] & 0x01) != 0)
			{
				flag |= 0x00010000;
				((int[])buffer[14])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}

			// config band width
			++ index;
			if((simulacrum[4] & 0x80) != 0)
			{
				flag |= 0x00008000;
				((int[])buffer[15])[0] = simulacrum[index++] & 0xff; // unsigned byte
			}
		}
		for(int i = 0; i < 5; ++i)
			fire(i);
		if((flag & 0x02000000) != 0)
			fire(5);
		if((flag & 0x01000000) != 0)
			fire(6);
		if((flag & 0x00800000) != 0)
			fire(7);
		if((flag & 0x00400000) != 0)
			fire(8);
		if((flag & 0x00200000) != 0)
			fire(9);
		if((flag & 0x00100000) != 0)
			fire(10);
		if((flag & 0x00080000) != 0)
			fire(11);
		if((flag & 0x00040000) != 0)
			fire(12);
		if((flag & 0x00020000) != 0)
			fire(13);
		if((flag & 0x00010000) != 0)
			fire(14);
		if((flag & 0x00008000) != 0)
			fire(15);
		mReadMotoringFlag = flag;
		return true;
	}
	
	@Override
	protected void notifyMotoringDataChanged(DeviceDataChangedListener listener, Robot robot, long timestamp)
	{
		Device[] devices = mDevices;
		Object[] buffer = mReadBuffer;
		int flag = mReadMotoringFlag;
		
		for(int i = 0; i < 5; ++i)
			listener.onDeviceDataChanged(robot, devices[i], buffer[i], timestamp);
		if((flag & 0x02000000) != 0)
			listener.onDeviceDataChanged(robot, devices[5], buffer[5], timestamp);
		if((flag & 0x01000000) != 0)
			listener.onDeviceDataChanged(robot, devices[6], buffer[6], timestamp);
		if((flag & 0x00800000) != 0)
			listener.onDeviceDataChanged(robot, devices[7], buffer[7], timestamp);
		if((flag & 0x00400000) != 0)
			listener.onDeviceDataChanged(robot, devices[8], buffer[8], timestamp);
		if((flag & 0x00200000) != 0)
			listener.onDeviceDataChanged(robot, devices[9], buffer[9], timestamp);
		if((flag & 0x00100000) != 0)
			listener.onDeviceDataChanged(robot, devices[10], buffer[10], timestamp);
		if((flag & 0x00080000) != 0)
			listener.onDeviceDataChanged(robot, devices[11], buffer[11], timestamp);
		if((flag & 0x00040000) != 0)
			listener.onDeviceDataChanged(robot, devices[12], buffer[12], timestamp);
		if((flag & 0x00020000) != 0)
			listener.onDeviceDataChanged(robot, devices[13], buffer[13], timestamp);
		if((flag & 0x00010000) != 0)
			listener.onDeviceDataChanged(robot, devices[14], buffer[14], timestamp);
		if((flag & 0x00008000) != 0)
			listener.onDeviceDataChanged(robot, devices[15], buffer[15], timestamp);
	}
	
	@Override
	protected byte[] encodeMotoringSimulacrum()
	{
		byte[] simulacrum = mMotoringSimulacrum;
		synchronized(mWriteLock)
		{
			Object[] buffer = mWriteBuffer;
			int flag = mWriteMotoringFlag;
			
			simulacrum[0] = (byte)0x00; // version
			simulacrum[1] = 0; // network id
			simulacrum[2] = (byte)0xfc;
			simulacrum[3] = (byte)0x00;
			simulacrum[4] = (byte)0x00;
			simulacrum[5] = (byte)0x00;

			// left wheel
			simulacrum[6] = (byte)(((int[])buffer[0])[0] & 0xff);

			// right wheel
			simulacrum[7] = (byte)(((int[])buffer[1])[0] & 0xff);

			// buzzer
			float[] t = (float[])buffer[2];
			int v = (int)(t[0] * 100);
			simulacrum[8] = (byte)((v >> 24) & 0xff);
			simulacrum[9] = (byte)((v >> 16) & 0xff);
			simulacrum[10] = (byte)((v >> 8) & 0xff);
			simulacrum[11] = (byte)(v & 0xff);

			// output a
			simulacrum[12] = (byte)(((int[])buffer[3])[0] & 0xff);

			// output b
			simulacrum[13] = (byte)(((int[])buffer[4])[0] & 0xff);

			int index = 14;
			// topology
			simulacrum[index++] = 0;
			if((flag & 0x02000000) != 0)
			{
				simulacrum[2] |= 0x02;
				simulacrum[index++] = (byte)(((int[])buffer[5])[0] & 0xff);
			}

			// left led
			simulacrum[index++] = 0;
			if((flag & 0x01000000) != 0)
			{
				simulacrum[2] |= 0x01;
				simulacrum[index++] = (byte)(((int[])buffer[6])[0] & 0xff);
			}

			// right led
			simulacrum[index++] = 0;
			if((flag & 0x00800000) != 0)
			{
				simulacrum[3] |= 0x80;
				simulacrum[index++] = (byte)(((int[])buffer[7])[0] & 0xff);
			}

			// note
			simulacrum[index++] = 0;
			if((flag & 0x00400000) != 0)
			{
				simulacrum[3] |= 0x40;
				simulacrum[index++] = (byte)(((int[])buffer[8])[0] & 0xff);
			}

			// line tracer mode
			simulacrum[index++] = 0;
			if((flag & 0x00200000) != 0)
			{
				simulacrum[3] |= 0x20;
				simulacrum[index++] = (byte)(((int[])buffer[9])[0] & 0xff);
			}

			// line tracer speed
			simulacrum[index++] = 0;
			if((flag & 0x00100000) != 0)
			{
				simulacrum[3] |= 0x10;
				simulacrum[index++] = (byte)(((int[])buffer[10])[0] & 0xff);
			}

			// io mode a
			simulacrum[index++] = 0;
			if((flag & 0x00080000) != 0)
			{
				simulacrum[3] |= 0x08;
				simulacrum[index++] = (byte)(((int[])buffer[11])[0] & 0xff);
			}

			// io mode b
			simulacrum[index++] = 0;
			if((flag & 0x00040000) != 0)
			{
				simulacrum[3] |= 0x04;
				simulacrum[index++] = (byte)(((int[])buffer[12])[0] & 0xff);
			}

			// config proximity
			simulacrum[index++] = 0;
			if((flag & 0x00020000) != 0)
			{
				simulacrum[3] |= 0x02;
				simulacrum[index++] = (byte)(((int[])buffer[13])[0] & 0xff);
			}

			// config gravity
			simulacrum[index++] = 0;
			if((flag & 0x00010000) != 0)
			{
				simulacrum[3] |= 0x01;
				simulacrum[index++] = (byte)(((int[])buffer[14])[0] & 0xff);
			}

			// config band width
			simulacrum[index++] = 0;
			if((flag & 0x00008000) != 0)
			{
				simulacrum[4] |= 0x80;
				simulacrum[index++] = (byte)(((int[])buffer[15])[0] & 0xff);
			}
			mWriteMotoringFlag = 0;
		}
		return simulacrum;
	}
}
