package org.roboid.robot.impl;

import java.util.ArrayList;

import org.roboid.robot.Device;
import org.roboid.robot.DeviceDataChangedListener;
import org.roboid.robot.Roboid;
import org.roboid.robot.Robot;

/**
 * @author akaii@kw.ac.kr (Kwang-Hyun Park)
 */
public abstract class RobotImpl extends NamedElementImpl implements Robot
{
	private final int mTag;
	private final int mIndex;
	private int mIcon;
	private final ArrayList<RoboidImpl> mRoboids = new ArrayList<RoboidImpl>();
	
	protected RobotImpl(int tag, int index)
	{
		mTag = tag;
		mIndex = index;
	}
	
	int getTag()
	{
		return mTag;
	}
	
	@Override
	public int getIndex()
	{
		return mIndex;
	}

	@Override
	public int getIcon()
	{
		return mIcon;
	}
	
	protected void setIcon(int icon)
	{
		mIcon = icon;
	}

	protected void addRoboid(RoboidImpl roboid)
	{
		if(roboid == null) return;
		if(mRoboids.contains(roboid)) return;
		mRoboids.add(roboid);
	}
	
	protected void updateDeviceState()
	{
		for(RoboidImpl roboid : mRoboids)
			roboid.updateDeviceState();
	}
	
	@Override
	public Device findDeviceById(int deviceId)
	{
		Device device = null;
		for(RoboidImpl roboid : mRoboids)
		{
			device = roboid.findDevice(deviceId);
			if(device != null)
				return device;
		}
		return null;
	}

	@Override
	public void addDeviceDataChangedListener(DeviceDataChangedListener listener)
	{
		if(listener == null) return;
		for(Roboid roboid : mRoboids)
			roboid.addDeviceDataChangedListener(listener);
	}

	@Override
	public void removeDeviceDataChangedListener(DeviceDataChangedListener listener)
	{
		if(listener == null) return;
		for(Roboid roboid : mRoboids)
			roboid.removeDeviceDataChangedListener(listener);
	}
	
	@Override
	public void clearDeviceDataChangedListeners()
	{
		for(Roboid roboid : mRoboids)
			roboid.clearDeviceDataChangedListeners();
	}
}
